﻿#property copyright "Copyright © 2009-2024, EarnForex.com"
#property link      "https://www.earnforex.com/"

#property description "Kicks in when position reaches at least TrailingStop pips of profit."

input double TrailingStop = 5;
input double StopLoss = 0; // Stop-Loss: Set to >= 0 to activate stop-loss.

void OnTick()
{
    double PointValue;
    int OrdersCount = OrdersTotal();
    for (int i = OrdersCount - 1; i >= 0; i--)
    {
        if (!OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
        {
            Print("Error selecting order: ", GetLastError());
            continue;
        }
        if (OrderSymbol() != Symbol()) continue; // Skipping positions in other currency pairs.
        // Calculate the point value in case there are extra digits in the quotes:
        if (MarketInfo(OrderSymbol(), MODE_POINT) == 0.00001) PointValue = 0.0001;
        else if (MarketInfo(OrderSymbol(), MODE_POINT) == 0.001) PointValue = 0.01;
        else PointValue = MarketInfo(OrderSymbol(), MODE_POINT);
        // Normalize trailing stop value to the point value:
        double TSTP = TrailingStop * PointValue;

        if (OrderType() == OP_BUY)
        {
            if (Bid - OrderOpenPrice() > TSTP)
            {
                if (OrderStopLoss() < Bid - TSTP)
                {
                    if (!OrderModify(OrderTicket(), OrderOpenPrice(), Bid - TSTP, OrderTakeProfit(), OrderExpiration(), clrNONE))
                    {
                        Print("Error setting Buy trailing stop: ", GetLastError());
                    }
                }
            }
            else if ((StopLoss != 0) && (OrderStopLoss() == 0) && (MathAbs(OrderStopLoss() - (Bid - StopLoss * PointValue)) > _Point / 2))
            {
                if (!OrderModify(OrderTicket(), OrderOpenPrice(), Bid - StopLoss * PointValue, OrderTakeProfit(), OrderExpiration(), clrNONE))
                {
                    Print("Error setting Buy stop-loss: ", GetLastError());
                }
            }
        }
        else if (OrderType() == OP_SELL)
        {
            if (OrderOpenPrice() - Ask > TSTP)
            {
                if ((OrderStopLoss() > Ask + TSTP) || (OrderStopLoss() == 0))
                {
                    if (!OrderModify(OrderTicket(), OrderOpenPrice(), Ask + TSTP, OrderTakeProfit(), OrderExpiration(), clrNONE))
                    {
                        Print("Error setting Sell trailing stop: ", GetLastError());
                    }
                }
            }
            else if ((StopLoss != 0) && (OrderStopLoss() == 0) && (MathAbs(OrderStopLoss() - (Ask + StopLoss * PointValue)) > _Point / 2))
            {
                if (!OrderModify(OrderTicket(), OrderOpenPrice(), Ask + StopLoss * PointValue, OrderTakeProfit(), OrderExpiration(), clrNONE))
                {
                    Print("Error setting Sell stop-loss: ", GetLastError());
                }
            }
        }
    }
}
//+------------------------------------------------------------------+